﻿namespace WarmlyLocomotove
{
    /// <summary>
    /// Класс, отвечающий за отрисовку объекта-сущности
    /// </summary>
    public class DrawningLocomotive
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityLocomotive Locomotive { get; set; }
        /// <summary>
        /// Класс отрисовки колёс
        /// </summary>
        public IDrawningAdditionalElements AdditionalElements;
        /// <summary>
        /// Левая координата отрисовки локомотива
        /// </summary>
        protected float _startPosX;
        /// <summary>
        /// Верхняя координата отрисовки локомотива
        /// </summary>
        protected float _startPosY;
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private int? _pictureWidth;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private int? _pictureHeight;
        /// <summary>
        /// Ширина локомотива
        /// </summary>
        protected readonly int _locomotiveWidth = 160;
        /// <summary>
        /// Высота локомотива
        /// </summary>
        protected readonly int _locomotiveHeight = 80;
        /// <summary>
        /// Инициализация свойств объекта-сущности
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Цвет кузова</param>
        public DrawningLocomotive(int speed, float weight, Color bodyColor)
        {
            Locomotive = new EntityLocomotive(speed, weight, bodyColor);
            AdditionalElements = new DrawningWheels();
        }
        /// <summary>
        /// Конструктор для изменения размеров локомотива
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Цвет кузова</param>
        /// <param name="locomotiveWidth">Ширина локомотива</param>
        /// <param name="locomotiveHeight">Высота локомотива</param>
        protected DrawningLocomotive(int speed, float weight, Color bodyColor, int locomotiveWidth, int locomotiveHeight)
            : this(speed, weight, bodyColor)
        {
            _locomotiveWidth = locomotiveWidth;
            _locomotiveHeight = locomotiveHeight;
        }
        /// <summary>
        /// Установка начальной позиции локомотива
        /// </summary>
        /// <param name="x">Левая координата</param>
        /// <param name="y">Правая координата</param>
        /// <param name="width">Ширина</param>
        /// <param name="height">Высота</param>
        public void SetPosition(int x, int y, int width, int height)
        {
            if (x < 0 || y < 0 || x + _locomotiveWidth > width || y + _locomotiveHeight > height)
            {
                return;
            }
            _startPosX = x;
            _startPosY = y;
            _pictureWidth = width;
            _pictureHeight = height;
        }
        /// <summary>
        /// Перемещение локомотива в зависимости от направления
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveLocomotive(Direction direction)
        {
            if (!_pictureWidth.HasValue || !_pictureHeight.HasValue)
            {
                return;
            }
            if (!_pictureWidth.HasValue || !_pictureHeight.HasValue)
            {
                return;
            }
            switch (direction)
            {
                //вправо
                case Direction.Right:
                    if (_startPosX + _locomotiveWidth + Locomotive.Step < _pictureWidth)
                    {
                        _startPosX += Locomotive.Step;
                    }
                    break;
                //влево
                case Direction.Left:
                    if (_startPosX >= Locomotive.Step)
                    {
                        _startPosX -= Locomotive.Step;
                    }
                    break;
                //вверх
                case Direction.Up:
                    if (_startPosY >= Locomotive.Step)
                    {
                        _startPosY -= Locomotive.Step;
                    }
                    break;
                //вниз
                case Direction.Down:
                    if (_startPosY + _locomotiveHeight + Locomotive.Step < _pictureHeight)
                    {
                        _startPosY += Locomotive.Step;
                    }
                    break;
            }
        }
        /// <summary>
        /// Метод отрисовки локомотива
        /// </summary>
        /// <param name="g">Графика</param>
        public virtual void DrawTransport(Graphics g)
        {
            if (_startPosX < 0 || _startPosY < 0 || !_pictureHeight.HasValue || !_pictureWidth.HasValue)
            {
                return;
            }
            //кузов
            Pen penBodyColor = new Pen(Locomotive?.BodyColor ?? Color.Black);
            for (int i = (int)_startPosX + 10; i <= _startPosX + 20; i++)
            {
                g.DrawLine(penBodyColor, _startPosX + 20, _startPosY, i, _startPosY + 20);
            }
            Brush brBodyColor = new SolidBrush(Locomotive?.BodyColor ?? Color.Black);
            g.FillRectangle(brBodyColor, _startPosX + 20, _startPosY, 130, 20);//верхняя часть
            g.FillRectangle(brBodyColor, _startPosX + 10, _startPosY + 20, 140, 20);//нижняя часть
            //переход
            Brush brBlack = new SolidBrush(Color.Black);
            g.FillRectangle(brBlack, _startPosX + 150, _startPosY + 5, 10, 30);
            Pen pen = new(Color.Black);
            //границы локомотива
            g.DrawLine(pen, _startPosX + 20, _startPosY, _startPosX + 10, _startPosY + 20);
            g.DrawLine(pen, _startPosX, _startPosY + 50, _startPosX + 10, _startPosY + 50);
            for (int i = (int)_startPosX + 10; i <= (int)_startPosX + 20; i++)
            {
                g.DrawLine(pen, i, _startPosY + 40, i - 10, _startPosY + 50);
            }
            for (int i = (int)_startPosX + 140; i <= (int)_startPosX + 150; i++)
            {
                g.DrawLine(pen, i, _startPosY + 40, i + 10, _startPosY + 50);
            }
            g.DrawRectangle(pen, _startPosX + 10, _startPosY + 20, 140, 20);
            g.DrawLine(pen, _startPosX + 20, _startPosY, _startPosX + 150, _startPosY);
            g.DrawLine(pen, _startPosX + 150, _startPosY, _startPosX + 150, _startPosY + 20);
            //окна
            Brush brWhite = new SolidBrush(Color.White);
            g.FillRectangle(brWhite, _startPosX + 25, _startPosY + 5, 10, 10);
            g.DrawRectangle(pen, _startPosX + 25, _startPosY + 5, 10, 10);
            g.FillRectangle(brWhite, _startPosX + 40, _startPosY + 5, 10, 10);
            g.DrawRectangle(pen, _startPosX + 40, _startPosY + 5, 10, 10);
            g.FillRectangle(brWhite, _startPosX + 130, _startPosY + 5, 10, 10);
            g.DrawRectangle(pen, _startPosX + 130, _startPosY + 5, 10, 10);
            //дверь
            g.FillRectangle(brBodyColor, _startPosX + 60, _startPosY + 10, 15, 25);
            g.DrawRectangle(pen, _startPosX + 60, _startPosY + 10, 15, 25);
            //колёса
            AdditionalElements.DrawWheels(g, _startPosX, _startPosY, Color.Black);
            AdditionalElements.DrawOrnament(g, _startPosX, _startPosY);
        }
        /// <summary>
        /// Метод перерисовки при изменении границ рисунка
        /// </summary>
        /// <param name="width">Новая ширина</param>
        /// <param name="height">Новая высота</param>
        public void ChangeBorders(int width, int height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_pictureWidth <= _locomotiveWidth || _pictureHeight <= _locomotiveHeight)
            {
                _pictureWidth = null;
                _pictureHeight = null;
                return;
            }
            if (_startPosX + _locomotiveWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth.Value - _locomotiveWidth;
            }
            if (_startPosY + _locomotiveHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight.Value - _locomotiveHeight;
            }
        }
        /// <summary>
        /// Получение текущей позиции объекта
        /// </summary>
        /// <returns></returns>
        public (float Top, float Bottom, float Left, float Right) GetCurrentPosition()
        {
            return (_startPosY, _startPosY + _locomotiveHeight, _startPosX, _startPosX + _locomotiveWidth);
        }
    }
}
